@@ -1,7 +1,12 @@
+import numpy as np
+import functools
+import logging
 from pathlib import Path
 
 from .exceptions import SABaseException
 
+logger = logging.getLogger("superannotate-python-sdk")
+
 _PROJECT_TYPES = {"Vector": 1, "Pixel": 2}
 _ANNOTATION_STATUSES = {
     "NotStarted": 1,
@@ -93,3 +98,67 @@ def annotation_status_int_to_str(annotation_status):
     raise SABaseException(
         0, "Annotation status should be one of 1, 2, 3, 4, 5, 6 ."
     )
+
+
+def deprecated_alias(**aliases):
+    def deco(f):
+        @functools.wraps(f)
+        def wrapper(*args, **kwargs):
+            rename_kwargs(f.__name__, kwargs, aliases)
+            return f(*args, **kwargs)
+
+        return wrapper
+
+    return deco
+
+
+def rename_kwargs(func_name, kwargs, aliases):
+    for alias, new in aliases.items():
+        if alias in kwargs:
+            if new in kwargs:
+                raise TypeError(
+                    '{} received both {} and {}'.format(func_name, alias, new)
+                )
+            logger.warning(
+                '%s is deprecated; use %s in %s', alias, new, func_name
+            )
+            kwargs[new] = kwargs.pop(alias)
+
+
+logger = logging.getLogger('httplogger')
+
+
+def hex_to_rgb(hex_string):
+    """Converts HEX values to RGB values
+    """
+    h = hex_string.lstrip('#')
+    return tuple(int(h[i:i + 2], 16) for i in (0, 2, 4))
+
+
+def rgb_to_hex(rgb_tuple):
+    """Converts RGB values to HEX values
+    """
+    return '#%02x%02x%02x' % rgb_tuple
+
+
+def blue_color_generator(n, hex_values=True):
+    """ Blue colors generator for SuperAnnotate blue mask.
+    """
+    hex_colors = []
+    for i in range(n + 1):
+        int_color = i * 15
+        bgr_color = np.array(
+            [
+                int_color & 255, (int_color >> 8) & 255,
+                (int_color >> 16) & 255, 255
+            ],
+            dtype=np.uint8
+        )
+        hex_color = '#' + "{:02x}".format(
+            bgr_color[2]
+        ) + "{:02x}".format(bgr_color[1], ) + "{:02x}".format(bgr_color[0])
+        if hex_values:
+            hex_colors.append(hex_color)
+        else:
+            hex_colors.append(hex_to_rgb(hex_color))
+    return hex_colors[1:]
