@@ -4,6 +4,9 @@ import logging
 from pathlib import Path
 
 import boto3
+import cv2
+import numpy as np
+from PIL import Image, ImageDraw
 import requests
 
 from ..annotation_helpers import (
@@ -13,9 +16,16 @@ from ..annotation_helpers import (
     add_annotation_template_to_json
 )
 from ..api import API
-from ..common import annotation_status_str_to_int
+from ..common import (
+    annotation_status_str_to_int, deprecated_alias, project_type_int_to_str,
+    image_path_to_annotation_paths, hex_to_rgb
+)
 from ..exceptions import SABaseException
-from .annotation_classes import search_annotation_classes
+from .annotation_classes import (
+    fill_class_and_attribute_ids, fill_class_and_attribute_names,
+    get_annotation_classes_id_to_name, get_annotation_classes_name_to_id,
+    search_annotation_classes
+)
 from .project import get_project_metadata
 
 logger = logging.getLogger("superannotate-python-sdk")
@@ -161,10 +171,17 @@ def set_image_annotation_status(project, image_name, annotation_status):
 
 
 def add_annotation_bbox_to_image(
-    project, image_name, bbox, annotation_class_name, error=None
+    project,
+    image_name,
+    bbox,
+    annotation_class_name,
+    annotation_class_attributes=None,
+    error=None,
 ):
     """Add a bounding box annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>" }, "groupName" : "<attribute_group>"} ], ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -173,12 +190,18 @@ def add_annotation_bbox_to_image(
     :type bbox: list of floats
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_bbox_to_json(
-        annotations, bbox, annotation_class_name, error
+        annotations,
+        bbox,
+        annotation_class_name,
+        annotation_class_attributes,
+        error,
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -186,10 +209,17 @@ def add_annotation_bbox_to_image(
 
 
 def add_annotation_polygon_to_image(
-    project, image_name, polygon, annotation_class_name, error=None
+    project,
+    image_name,
+    polygon,
+    annotation_class_name,
+    annotation_class_attributes=None,
+    error=None
 ):
     """Add a polygon annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>", "groupName" : "<attribute_group>"},  ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -198,13 +228,16 @@ def add_annotation_polygon_to_image(
     :type polygon: list of floats
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
 
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_polygon_to_json(
-        annotations, polygon, annotation_class_name, error
+        annotations, polygon, annotation_class_name,
+        annotation_class_attributes, error
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -212,10 +245,17 @@ def add_annotation_polygon_to_image(
 
 
 def add_annotation_polyline_to_image(
-    project, image_name, polyline, annotation_class_name, error=None
+    project,
+    image_name,
+    polyline,
+    annotation_class_name,
+    annotation_class_attributes=None,
+    error=None
 ):
     """Add a polyline annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>", "groupName" : "<attribute_group>"},  ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -224,12 +264,15 @@ def add_annotation_polyline_to_image(
     :type polyline: list of floats
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_polyline_to_json(
-        annotations, polyline, annotation_class_name, error
+        annotations, polyline, annotation_class_name,
+        annotation_class_attributes, error
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -237,10 +280,17 @@ def add_annotation_polyline_to_image(
 
 
 def add_annotation_point_to_image(
-    project, image_name, point, annotation_class_name, error=None
+    project,
+    image_name,
+    point,
+    annotation_class_name,
+    annotation_class_attributes=None,
+    error=None
 ):
     """Add a point annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>", "groupName" : "<attribute_group>"},  ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -249,12 +299,15 @@ def add_annotation_point_to_image(
     :type point: list of floats
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_point_to_json(
-        annotations, point, annotation_class_name, error
+        annotations, point, annotation_class_name, annotation_class_attributes,
+        error
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -262,10 +315,17 @@ def add_annotation_point_to_image(
 
 
 def add_annotation_ellipse_to_image(
-    project, image_name, ellipse, annotation_class_name, error=None
+    project,
+    image_name,
+    ellipse,
+    annotation_class_name,
+    annotation_class_attributes=None,
+    error=None
 ):
     """Add an ellipse annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>", "groupName" : "<attribute_group>"},  ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -274,12 +334,15 @@ def add_annotation_ellipse_to_image(
     :type ellipse: list of floats
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_ellipse_to_json(
-        annotations, ellipse, annotation_class_name, error
+        annotations, ellipse, annotation_class_name,
+        annotation_class_attributes, error
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -292,10 +355,13 @@ def add_annotation_template_to_image(
     template_points,
     template_connections,
     annotation_class_name,
+    annotation_class_attributes=None,
     error=None
 ):
     """Add a template annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>", "groupName" : "<attribute_group>"},  ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -310,13 +376,15 @@ def add_annotation_template_to_image(
     :type template_connections: list of ints
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_template_to_json(
         annotations, template_points, template_connections,
-        annotation_class_name, error
+        annotation_class_name, annotation_class_attributes, error
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -324,10 +392,17 @@ def add_annotation_template_to_image(
 
 
 def add_annotation_cuboid_to_image(
-    project, image_name, cuboid, annotation_class_name, error=None
+    project,
+    image_name,
+    cuboid,
+    annotation_class_name,
+    annotation_class_attributes=None,
+    error=None
 ):
     """Add a cuboid annotation to image annotations
 
+    annotation_class_attributes has the form [ {"name" : "<attribute_value>", "groupName" : "<attribute_group>"},  ... ]
+
     :param project: project name or metadata of the project
     :type project: str or dict
     :param image_name: image name
@@ -339,12 +414,15 @@ def add_annotation_cuboid_to_image(
     :type cuboid: list of floats
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
+    :param annotation_class_attributes: list of annotation class attributes
+    :type annotation_class_attributes: list of 2 element dicts
     :param error: if not None, marks annotation as error (True) or no-error (False)
     :type error: bool
     """
     annotations = get_image_annotations(project, image_name)["annotation_json"]
     annotations = add_annotation_cuboid_to_json(
-        annotations, cuboid, annotation_class_name, error
+        annotations, cuboid, annotation_class_name, annotation_class_attributes,
+        error
     )
     upload_annotations_from_json_to_image(
         project, image_name, annotations, verbose=False
@@ -356,6 +434,7 @@ def download_image(
     image_name,
     local_dir_path=".",
     include_annotations=False,
+    include_fuse=False,
     variant='original'
 ):
     """Downloads the image (and annotation if not None) to local_dir_path
@@ -368,6 +447,8 @@ def download_image(
     :type local_dir_path: Pathlike (str or Path)
     :param include_annotations: enables annotation download with the image
     :type include_annotations: bool
+    :param include_fuse: enables fuse image download with the image
+    :type include_fuse: bool
     :param variant: which resolution to download, can be 'original' or 'lores'
      (low resolution used in web editor)
     :type variant: str
@@ -379,6 +460,13 @@ def download_image(
         raise SABaseException(
             0, f"local_dir_path {local_dir_path} is not an existing directory"
         )
+    if variant not in ["original", "lores"]:
+        raise SABaseException(
+            0, "Image download variant should be either original or lores"
+        )
+
+    if not isinstance(project, dict):
+        project = get_project_metadata(project)
     img = get_image_bytes(project, image_name, variant=variant)
     if variant == "lores":
         image_name += "___lores.jpg"
@@ -386,13 +474,18 @@ def download_image(
     with open(filepath, 'wb') as f:
         f.write(img.getbuffer())
     annotations_filepaths = None
+    fuse_path = None
     if include_annotations:
         annotations_filepaths = download_image_annotations(
             project, image_name, local_dir_path
         )
+        if include_fuse:
+            classes = search_annotation_classes(project, return_metadata=True)
+            project_type = project_type_int_to_str(project["type"])
+            fuse_path = create_fuse_image(filepath, classes, project_type)
     logger.info("Downloaded image %s to %s.", image_name, filepath)
 
-    return (str(filepath), annotations_filepaths)
+    return (str(filepath), annotations_filepaths, fuse_path)
 
 
 def delete_image(project, image_name):
@@ -432,6 +525,10 @@ def get_image_bytes(project, image_name, variant='original'):
     :return: io.BytesIO() of the image
     :rtype: io.BytesIO()
     """
+    if variant not in ["original", "lores"]:
+        raise SABaseException(
+            0, "Image download variant should be either original or lores"
+        )
     image = get_image_metadata(project, image_name)
     team_id, project_id, image_id, folder_id = image["team_id"], image[
         "project_id"], image["id"], image['folder_id']
@@ -495,9 +592,9 @@ def get_image_preannotations(project, image_name):
     annotation_classes = search_annotation_classes(
         project, return_metadata=True
     )
-    annotation_classes_dict = {}
-    for annotation_class in annotation_classes:
-        annotation_classes_dict[annotation_class["id"]] = annotation_class
+    annotation_classes_dict = get_annotation_classes_id_to_name(
+        annotation_classes
+    )
     if project_type == 1:  # vector
         res = res['preannotation']
         url = res["url"]
@@ -505,16 +602,15 @@ def get_image_preannotations(project, image_name):
         headers = res["headers"]
         response = requests.get(url=url, headers=headers)
         if not response.ok:
-            logger.warning("No preannotation available for image %s.", image_id)
+            logger.warning(
+                "No preannotation available for image %s.", image_name
+            )
             return {
                 "preannotation_json_filename": None,
                 "preannotation_json": None
             }
         res_json = response.json()
-        for r in res_json:
-            if "classId" in r and r["classId"] in annotation_classes_dict:
-                r["className"] = annotation_classes_dict[r["classId"]]["name"]
-
+        fill_class_and_attribute_names(res_json, annotation_classes_dict)
         return {
             "preannotation_json_filename": annotation_json_filename,
             "preannotation_json": res_json
@@ -534,9 +630,9 @@ def get_image_preannotations(project, image_name):
                 "preannotation_mask": None,
             }
         preannotation_json = response.json()
-        for r in preannotation_json:
-            if "classId" in r and r["classId"] in annotation_classes_dict:
-                r["className"] = annotation_classes_dict[r["classId"]]["name"]
+        fill_class_and_attribute_names(
+            preannotation_json, annotation_classes_dict
+        )
 
         res_mask = res['preAnnotationSavePng']
         url = res_mask["url"]
@@ -592,9 +688,9 @@ def get_image_annotations(project, image_name, project_type=None):
     annotation_classes = search_annotation_classes(
         project, return_metadata=True
     )
-    annotation_classes_dict = {}
-    for annotation_class in annotation_classes:
-        annotation_classes_dict[annotation_class["id"]] = annotation_class
+    annotation_classes_dict = get_annotation_classes_id_to_name(
+        annotation_classes
+    )
     if project_type == 1:  # vector
         url = res["objects"]["url"]
         annotation_json_filename = url.rsplit('/', 1)[-1]
@@ -602,10 +698,7 @@ def get_image_annotations(project, image_name, project_type=None):
         response = requests.get(url=url, headers=headers)
         if response.ok:
             res_json = response.json()
-            for r in res_json:
-                if "classId" in r and r["classId"] in annotation_classes_dict:
-                    r["className"] = annotation_classes_dict[r["classId"]
-                                                            ]["name"]
+            fill_class_and_attribute_names(res_json, annotation_classes_dict)
             return {
                 "annotation_json_filename": annotation_json_filename,
                 "annotation_json": res_json
@@ -628,9 +721,7 @@ def get_image_annotations(project, image_name, project_type=None):
         elif not response.ok:
             raise SABaseException(response.status_code, response.text)
         res_json = response.json()
-        for r in res_json:
-            if "classId" in r and r["classId"] in annotation_classes_dict:
-                r["className"] = annotation_classes_dict[r["classId"]]["name"]
+        fill_class_and_attribute_names(res_json, annotation_classes_dict)
         url = res["pixelSave"]["url"]
         annotation_mask_filename = url.rsplit('/', 1)[-1]
         headers = res["pixelSave"]["headers"]
@@ -708,28 +799,26 @@ def download_image_preannotations(project, image_name, local_dir_path):
         return (None, )
     return_filepaths = []
     json_path = Path(local_dir_path) / annotation["preannotation_json_filename"]
-    return_filepaths.append(json_path)
+    return_filepaths.append(str(json_path))
     if project["type"] == 1:
         with open(json_path, "w") as f:
             json.dump(annotation["preannotation_json"], f)
     else:
-        with open(
-            Path(local_dir_path) / annotation["preannotation_json_filename"],
-            "w"
-        ) as f:
+        with open(json_path, "w") as f:
             json.dump(annotation["preannotation_json"], f)
-        with open(
-            Path(local_dir_path) / annotation["preannotation_mask_filename"],
-            "wb"
-        ) as f:
+        mask_path = Path(local_dir_path
+                        ) / annotation["preannotation_mask_filename"]
+        with open(mask_path, "wb") as f:
             f.write(annotation["preannotation_mask"].getbuffer())
+        return_filepaths.append(str(mask_path))
     return tuple(return_filepaths)
 
 
+@deprecated_alias(mask_path="mask")
 def upload_annotations_from_json_to_image(
-    project, image_name, annotation_json, mask_path=None, verbose=True
+    project, image_name, annotation_json, mask=None, verbose=True
 ):
-    """Upload annotations from JSON (also mask_path for pixel annotations)
+    """Upload annotations from JSON (also mask for pixel annotations)
     to the image.
 
     :param project: project name or metadata of the project
@@ -738,8 +827,8 @@ def upload_annotations_from_json_to_image(
     :type image: str
     :param annotation_json: annotations in SuperAnnotate format JSON dict or path to JSON file
     :type annotation_json: dict or Pathlike (str or Path)
-    :param mask_path: filepath to mask annotation for pixel projects in SuperAnnotate format
-    :type mask_path: Pathlike (str or Path)
+    :param mask: BytesIO object or filepath to mask annotation for pixel projects in SuperAnnotate format
+    :type mask: BytesIO or Pathlike (str or Path)
     """
 
     if not isinstance(annotation_json, list):
@@ -761,24 +850,10 @@ def upload_annotations_from_json_to_image(
     annotation_classes = search_annotation_classes(
         project, return_metadata=True
     )
-    annotation_classes_dict = {}
-    for annotation_class in annotation_classes:
-        if annotation_class["name"] in annotation_classes_dict:
-            logger.warning(
-                "Duplicate annotation class name %s. Only one of the annotation classes will be used. This will result in errors in annotation upload.",
-                annotation_class["name"]
-            )
-        annotation_classes_dict[annotation_class["name"]] = annotation_class
-    for ann in annotation_json:
-        if "userId" in ann and ann["type"] == "meta":
-            continue
-        annotation_class_name = ann["className"]
-        if not annotation_class_name in annotation_classes_dict:
-            raise SABaseException(
-                0, "Couldn't find annotation class " + annotation_class_name
-            )
-        class_id = annotation_classes_dict[annotation_class_name]["id"]
-        ann["classId"] = class_id
+    annotation_classes_dict = get_annotation_classes_name_to_id(
+        annotation_classes
+    )
+    fill_class_and_attribute_ids(annotation_json, annotation_classes_dict)
     params = {
         'team_id': team_id,
         'project_id': project_id,
@@ -804,8 +879,11 @@ def upload_annotations_from_json_to_image(
                 Key=res['filePath'], Body=json.dumps(annotation_json)
             )
         else:  # pixel
-            if mask_path is None:
+            if mask is None:
                 raise SABaseException(0, "Pixel annotation should have mask.")
+            if not isinstance(mask, io.BytesIO):
+                with open(mask, "rb") as f:
+                    mask = io.BytesIO(f.read())
             res_j = res['pixel']
             s3_session = boto3.Session(
                 aws_access_key_id=res_j['accessKeyId'],
@@ -825,8 +903,123 @@ def upload_annotations_from_json_to_image(
             )
             s3_resource = s3_session.resource('s3')
             bucket = s3_resource.Bucket(res_m["bucket"])
-            bucket.put_object(Key=res_m['filePath'], Body=open(mask_path, 'rb'))
+            bucket.put_object(Key=res_m['filePath'], Body=mask)
     else:
         raise SABaseException(
             response.status_code, "Couldn't upload annotation. " + response.text
         )
+
+
+def create_fuse_image(image, classes_json, project_type, in_memory=False):
+    """Creates fuse for locally located image and annotations
+
+    :param image: path to image
+    :type image: str or Pathlike
+    :param image_name: annotation classes or path to their JSON
+    :type image: list or Pathlike
+    :param project_type: project type, "Vector" or "Pixel"
+    :type project_type: str
+    :param in_memory: enables pillow Image return instead of saving the image
+    :type in_memory: bool
+
+    :return: path to created fuse image or pillow Image object if in_memory enabled
+    :rtype: str of PIL.Image
+    """
+    annotation_path = image_path_to_annotation_paths(image, project_type)
+    annotation_json = json.load(open(annotation_path[0]))
+    if not isinstance(classes_json, list):
+        classes_json = json.load(open(classes_json))
+    class_color_dict = {}
+    for ann_class in classes_json:
+        if "name" not in ann_class:
+            continue
+        class_color_dict[ann_class["name"]] = ann_class["color"]
+    image_size = Image.open(image).size
+    fi = np.full((image_size[1], image_size[0], 4), [0, 0, 0, 255], np.uint8)
+    if project_type == "Vector":
+        fi_pil = Image.fromarray(fi)
+        draw = ImageDraw.Draw(fi_pil)
+        for annotation in annotation_json:
+            if "className" not in annotation:
+                continue
+            color = class_color_dict[annotation["className"]]
+            rgb = hex_to_rgb(color)
+            fill_color = (rgb[0], rgb[1], rgb[2], 255)
+            outline_color = (255, 255, 255, 255)
+            if annotation["type"] == "bbox":
+                pt = (
+                    (annotation["points"]["x1"], annotation["points"]["y1"]),
+                    (annotation["points"]["x2"], annotation["points"]["y2"])
+                )
+                draw.rectangle(pt, fill_color, outline_color)
+            elif annotation["type"] == "polygon":
+                pts = annotation["points"]
+                draw.polygon(pts, fill_color, outline_color)
+            elif annotation["type"] == "polyline":
+                pts = annotation["points"]
+                draw.line(pts, fill_color, width=2)
+            elif annotation["type"] == "point":
+                pts = [
+                    annotation["x"] - 2, annotation["y"] - 2,
+                    annotation["x"] + 2, annotation["y"] + 2
+                ]
+                draw.ellipse(pts, fill_color, outline_color)
+            elif annotation["type"] == "ellipse":
+                temp = np.full(
+                    (image_size[1], image_size[0], 3), [0, 0, 0], np.uint8
+                )
+                cv2.ellipse(
+                    temp, (annotation["cx"], annotation["cy"]),
+                    (annotation["rx"], annotation["ry"]), annotation["angle"],
+                    0, 360, outline_color[:-1], 1
+                )
+                temp_mask = np.zeros(
+                    (image_size[1] + 2, image_size[0] + 2), np.uint8
+                )
+                cv2.floodFill(
+                    temp, temp_mask, (annotation["cx"], annotation["cy"]),
+                    fill_color[:-1]
+                )
+                temp_mask = np.alltrue(temp != [0, 0, 0],
+                                       axis=2).astype(np.uint8) * 255
+                # print(temp_mask.shape, temp_mask.dtype, np.max(temp_mask))
+                new_array = np.array(fi_pil)
+                new_array[:, :, :-1] += temp
+                new_array[:, :, 3] += temp_mask
+                fi_pil = Image.fromarray(new_array)
+                draw = ImageDraw.Draw(fi_pil)
+            elif annotation["type"] == "template":
+                pts = annotation["points"]
+                pt_dict = {}
+                for pt in pts:
+                    pt_e = [pt["x"] - 2, pt["y"] - 2, pt["x"] + 2, pt["y"] + 2]
+                    draw.ellipse(pt_e, fill_color, fill_color)
+                    pt_dict[pt["id"]] = [pt["x"], pt["y"]]
+                connections = annotation["connections"]
+                for connection in connections:
+                    draw.line(
+                        pt_dict[connection["from"]] + pt_dict[connection["to"]],
+                        fill_color,
+                        width=1
+                    )
+    else:
+        annotation_mask = np.array(Image.open(annotation_path[1]))
+        print(annotation_mask.shape, annotation_mask.dtype)
+        for annotation in annotation_json:
+            if "className" not in annotation or "parts" not in annotation:
+                continue
+            color = class_color_dict[annotation["className"]]
+            rgb = hex_to_rgb(color)
+            fill_color = (rgb[0], rgb[1], rgb[2], 255)
+            for part in annotation["parts"]:
+                part_color = part["color"]
+                part_color = list(hex_to_rgb(part_color)) + [255]
+                temp_mask = np.alltrue(annotation_mask == part_color, axis=2)
+                fi[temp_mask] = fill_color
+        fi_pil = Image.fromarray(fi)
+
+    if in_memory:
+        return fi_pil
+    fuse_path = str(image) + "___fuse.png"
+    fi_pil.save(fuse_path)
+    return fuse_path
