@@ -5,6 +5,7 @@ import math
 import sys
 import threading
 import time
+import copy
 from pathlib import Path
 
 import boto3
@@ -14,13 +15,17 @@ from tqdm import tqdm
 from ..api import API
 from ..common import (
     annotation_status_str_to_int, project_type_int_to_str,
-    project_type_str_to_int, user_role_str_to_int, user_role_int_to_str
+    project_type_str_to_int, user_role_int_to_str, user_role_str_to_int
 )
 from ..exceptions import (
     SABaseException, SAExistingProjectNameException,
     SANonExistingProjectNameException
 )
-from .annotation_classes import search_annotation_classes
+from .annotation_classes import (
+    create_annotation_classes_from_classes_json, search_annotation_classes,
+    get_annotation_classes_id_to_name, get_annotation_classes_name_to_id,
+    fill_class_and_attribute_ids, fill_class_and_attribute_names
+)
 from .project import get_project_metadata
 from .users import get_team_contributor_metadata
 
@@ -77,6 +82,85 @@ def create_project(project_name, project_description, project_type):
     return res
 
 
+def create_project_like_project(
+    project_name,
+    from_project,
+    project_description=None,
+    copy_annotation_classes=True,
+    copy_settings=True,
+    copy_workflow=True,
+    copy_project_contributors=False
+):
+    """Create a new project in the team using annotation classes and settings from from_project.
+
+    :param project_name: new project's name
+    :type project_name: str
+    :param from_project: the name or metadata of the project being used for duplication
+    :type from_project: str or dict
+    :param project_description: the new project's description. If None, from_project's
+                                description will be used
+    :type project_description: str
+    :param copy_annotation_classes: enables copying annotation classes
+    :type copy_annotation_classes: bool
+    :param copy_settings: enables copying project settings
+    :type copy_settings: bool
+    :param copy_workflow: enables copying project workflow
+    :type copy_workflow: bool
+    :param copy_project_contributors: enables copying project contributors
+    :type copy_project_contributors: bool
+
+    :return: dict object metadata of the new project
+    :rtype: dict
+    """
+    try:
+        get_project_metadata(project_name)
+    except SANonExistingProjectNameException:
+        pass
+    else:
+        raise SAExistingProjectNameException(
+            0, "Project with name " + project_name +
+            " already exists. Please use unique names for projects to use with SDK."
+        )
+    if not isinstance(from_project, dict):
+        from_project = get_project_metadata(from_project)
+    if project_description is None:
+        project_description = from_project["description"]
+    data = {
+        "team_id": str(_api.team_id),
+        "name": project_name,
+        "description": project_description,
+        "status": 0,
+        "type": from_project["type"]
+    }
+    response = _api.send_request(
+        req_type='POST', path='/project', json_req=data
+    )
+    if not response.ok:
+        raise SABaseException(
+            response.status_code, "Couldn't create project " + response.text
+        )
+    res = response.json()
+    logger.info(
+        "Created project %s (ID %s) with type %s", res["name"], res["id"],
+        project_type_int_to_str(res["type"])
+    )
+    if copy_settings:
+        set_project_settings(res, get_project_settings(from_project))
+    if copy_annotation_classes:
+        create_annotation_classes_from_classes_json(
+            res, search_annotation_classes(from_project, return_metadata=True)
+        )
+        if copy_workflow:
+            set_project_workflow(res, get_project_workflow(from_project))
+    if copy_project_contributors:
+        for user in from_project["users"]:
+            share_project(
+                res, user["user_id"], user_role_int_to_str(user["user_role"])
+            )
+
+    return res
+
+
 def delete_project(project):
     """Deletes the project
 
@@ -194,8 +278,8 @@ def upload_images_from_folder_to_project(
     :param image_quality_in_editor: image quality (in percents) that will be seen in SuperAnnotate web annotation editor. If None default value will be used.
     :type image_quality_in_editor: int
 
-    :return: uploaded images' filepaths
-    :rtype: list
+    :return: uploaded and not-uploaded images' filepaths
+    :rtype: tuple of list of strs
     """
     if not isinstance(project, dict):
         project = get_project_metadata(project)
@@ -205,7 +289,6 @@ def upload_images_from_folder_to_project(
         )
     if exclude_file_patterns is None:
         exclude_file_patterns = ["___save.png", "___fuse.png"]
-    project_id = project["id"]
     if extensions is None:
         extensions = ["jpg", "png"]
     elif not isinstance(extensions, list):
@@ -260,36 +343,50 @@ def upload_images_from_folder_to_project(
 def get_image_array_to_upload(
     byte_io_orig, project_type, image_quality_in_editor
 ):
+    Image.MAX_IMAGE_PIXELS = None
     im = Image.open(byte_io_orig)
+    im = im.convert("RGB")
     width, height = im.size
     max_size = _RESIZE_CONFIG[project_type]
     if (width * height) > max_size:
+        im_format = im.format
+        logger.warning("One of the images is being resized to smaller size.")
         max_size_root = math.sqrt(max_size)
         nwidth = math.floor(max_size_root * math.sqrt(width / height))
         nheight = math.floor(max_size_root * math.sqrt(height / width))
-        print(nwidth, nheight)
         im = im.resize((nwidth, nheight))
         byte_io_orig = io.BytesIO()
-        im.convert('RGB').save(
-            byte_io_orig, im.format, subsampling=0, quality=100
-        )
+        im.save(byte_io_orig, im_format, subsampling=0, quality=100)
 
     byte_io_lores = io.BytesIO()
-    im.convert('RGB').save(
+    im.save(
         byte_io_lores,
         'JPEG',
         subsampling=0 if image_quality_in_editor > 60 else 2,
         quality=image_quality_in_editor
     )
 
+    byte_io_huge = io.BytesIO()
+    hsize = int(height * 600.0 / width)
+    im.resize((600, hsize), Image.ANTIALIAS).save(byte_io_huge, 'JPEG')
+
     byte_io_thumbs = io.BytesIO()
-    im.convert('RGB').resize((128, 96)).save(byte_io_thumbs, 'JPEG')
+    thumbnail_size = (128, 96)
+    background = Image.new('RGB', thumbnail_size, "black")
+    im.thumbnail(thumbnail_size)
+    (w, h) = im.size
+    background.paste(
+        im, ((thumbnail_size[0] - w) // 2, (thumbnail_size[1] - h) // 2)
+    )
+    im = background
+    im.save(byte_io_thumbs, 'JPEG')
 
     byte_io_thumbs.seek(0)
     byte_io_lores.seek(0)
+    byte_io_huge.seek(0)
     byte_io_orig.seek(0)
 
-    return byte_io_orig, byte_io_lores, byte_io_thumbs
+    return byte_io_orig, byte_io_lores, byte_io_huge, byte_io_thumbs
 
 
 def __upload_images_to_aws_thread(
@@ -300,8 +397,8 @@ def __upload_images_to_aws_thread(
     prefix,
     thread_id,
     chunksize,
-    already_uploaded,
-    num_uploaded,
+    couldnt_upload,
+    uploaded,
     image_quality_in_editor,
     from_s3_bucket=None,
 ):
@@ -326,8 +423,6 @@ def __upload_images_to_aws_thread(
     for i in range(start_index, end_index):
         if i >= len_img_paths:
             break
-        if already_uploaded[i]:
-            continue
         path = img_paths[i]
         key = prefix + f'{Path(path).name}'
         if from_s3_bucket is not None:
@@ -337,32 +432,25 @@ def __upload_images_to_aws_thread(
         else:
             with open(path, "rb") as f:
                 file = io.BytesIO(f.read())
-        orig_image, lores_image, thumbnail_image = get_image_array_to_upload(
-            file, project_type, image_quality_in_editor
-        )
         try:
+            orig_image, lores_image, huge_image, thumbnail_image = get_image_array_to_upload(
+                file, project_type, image_quality_in_editor
+            )
             bucket.put_object(Body=orig_image, Key=key)
-        except Exception as e:
-            logger.warning("Unable to upload orig_image to data server %s", e)
-            break
-        try:
             bucket.put_object(Body=lores_image, Key=key + '___lores.jpg')
-        except Exception as e:
-            logger.warning("Unable to upload lores_image to data server %s.", e)
-            break
-        try:
+            bucket.put_object(Body=huge_image, Key=key + '___huge.jpg')
             bucket.put_object(Body=thumbnail_image, Key=key + '___thumb.jpg')
         except Exception as e:
-            logger.warning(
-                "Unable to upload thumbnail_image to data server %s.", e
-            )
-            break
-        num_uploaded[thread_id] += 1
-        already_uploaded[i] = True
-        uploaded_imgs.append(path)
-        if len(uploaded_imgs) >= 100:
-            __create_image(uploaded_imgs, project, annotation_status, prefix)
-            uploaded_imgs = []
+            logger.warning("Unable to upload to data server %s.", e)
+            couldnt_upload[thread_id].append(path)
+        else:
+            uploaded[thread_id].append(path)
+            uploaded_imgs.append(path)
+            if len(uploaded_imgs) >= 100:
+                __create_image(
+                    uploaded_imgs, project, annotation_status, prefix
+                )
+                uploaded_imgs = []
     __create_image(uploaded_imgs, project, annotation_status, prefix)
 
 
@@ -412,8 +500,8 @@ def upload_images_to_project(
     :param image_quality_in_editor: image quality (in percents) that will be seen in SuperAnnotate web annotation editor. If None default value will be used.
     :type image_quality_in_editor: int
 
-    :return: uploaded images' filepaths
-    :rtype: list of str
+    :return: uploaded and not-uploaded images' filepaths
+    :rtype: tuple of list of strs
     """
     if not isinstance(project, dict):
         project = get_project_metadata(project)
@@ -432,55 +520,64 @@ def upload_images_to_project(
     params = {
         'team_id': team_id,
     }
-    num_uploaded = [0] * _NUM_THREADS
-    already_uploaded = [False] * len_img_paths
+    uploaded = []
+    for _ in range(_NUM_THREADS):
+        uploaded.append([])
+    couldnt_upload = []
+    for _ in range(_NUM_THREADS):
+        couldnt_upload.append([])
     finish_event = threading.Event()
     chunksize = int(math.ceil(len(img_paths) / _NUM_THREADS))
     tqdm_thread = threading.Thread(
-        target=__tqdm_thread, args=(len_img_paths, num_uploaded, finish_event)
+        target=__tqdm_thread_image_upload,
+        args=(len_img_paths, uploaded, couldnt_upload, finish_event)
     )
     tqdm_thread.start()
-    while True:
-        if sum(num_uploaded) == len_img_paths:
-            break
-        response = _api.send_request(
-            req_type='GET',
-            path=f'/project/{project_id}/sdkImageUploadToken',
-            params=params
+    response = _api.send_request(
+        req_type='GET',
+        path=f'/project/{project_id}/sdkImageUploadToken',
+        params=params
+    )
+    if response.ok:
+        res = response.json()
+        prefix = res['filePath']
+    else:
+        raise SABaseException(
+            response.status_code, "Couldn't get upload token " + response.text
         )
-        if response.ok:
-            res = response.json()
-            prefix = res['filePath']
-        else:
-            raise SABaseException(
-                response.status_code,
-                "Couldn't get upload token " + response.text
-            )
 
-        threads = []
-        for thread_id in range(_NUM_THREADS):
-            t = threading.Thread(
-                target=__upload_images_to_aws_thread,
-                args=(
-                    res, img_paths, project, annotation_status, prefix,
-                    thread_id, chunksize, already_uploaded, num_uploaded,
-                    image_quality_in_editor, from_s3_bucket
-                )
+    threads = []
+    for thread_id in range(_NUM_THREADS):
+        t = threading.Thread(
+            target=__upload_images_to_aws_thread,
+            args=(
+                res, img_paths, project, annotation_status, prefix, thread_id,
+                chunksize, couldnt_upload, uploaded, image_quality_in_editor,
+                from_s3_bucket
             )
-            threads.append(t)
-            t.start()
-        for t in threads:
-            t.join()
+        )
+        threads.append(t)
+        t.start()
+    for t in threads:
+        t.join()
     finish_event.set()
     tqdm_thread.join()
+    list_of_not_uploaded = []
+    for couldnt_upload_thread in couldnt_upload:
+        for file in couldnt_upload_thread:
+            logger.warning("Couldn't upload image %s", file)
+            list_of_not_uploaded.append(str(file))
+    list_of_uploaded = []
+    for upload_thread in uploaded:
+        for file in upload_thread:
+            list_of_uploaded.append(str(file))
 
-    return_paths = [str(path) for path in img_paths]
-    return return_paths
+    return (list_of_uploaded, list_of_not_uploaded)
 
 
 def __upload_annotations_thread(
     team_id, project_id, project_type, anns_filenames, folder_path,
-    annotation_classes, thread_id, chunksize, num_uploaded, from_s3_bucket,
+    annotation_classes_dict, thread_id, chunksize, num_uploaded, from_s3_bucket,
     actually_uploaded
 ):
     NUM_TO_SEND = 500
@@ -489,14 +586,6 @@ def __upload_annotations_thread(
     if start_index >= len_anns:
         return
     end_index = min(start_index + chunksize, len_anns)
-    annotation_classes_dict = {}
-    for annotation_class in annotation_classes:
-        if annotation_class["name"] in annotation_classes_dict:
-            logger.warning(
-                "Duplicate annotation class name %s. Only one of the annotation classes will be used. This will result in errors in annotation upload.",
-                annotation_class["name"]
-            )
-        annotation_classes_dict[annotation_class["name"]] = annotation_class
 
     postfix_json = '___objects.json' if project_type == 1 else '___pixel.json'
     len_postfix_json = len(postfix_json)
@@ -544,20 +633,9 @@ def __upload_annotations_thread(
                 file.seek(0)
                 annotation_json = json.load(file)
 
-            for ann in annotation_json:
-                if (
-                    "userId" in ann and "type" in ann and ann["type"] == "meta"
-                ) or "className" not in ann:
-                    continue
-                annotation_class_name = ann["className"]
-                if not annotation_class_name in annotation_classes_dict:
-                    logger.error(
-                        "Couldn't find annotation class %s in project's annotation classes.",
-                        annotation_class_name
-                    )
-                    sys.exit(1)
-                class_id = annotation_classes_dict[annotation_class_name]["id"]
-                ann["classId"] = class_id
+            fill_class_and_attribute_ids(
+                annotation_json, annotation_classes_dict
+            )
             bucket.put_object(
                 Key=image_path + postfix_json, Body=json.dumps(annotation_json)
             )
@@ -704,6 +782,9 @@ def _upload_annotations_from_folder_to_project(
     annotation_classes = search_annotation_classes(
         project, return_metadata=True
     )
+    annotation_classes_dict = get_annotation_classes_name_to_id(
+        annotation_classes
+    )
     chunksize = int(math.ceil(len_annotations_paths / _NUM_THREADS))
     threads = []
     for thread_id in range(_NUM_THREADS):
@@ -711,7 +792,7 @@ def _upload_annotations_from_folder_to_project(
             target=__upload_annotations_thread,
             args=(
                 team_id, project_id, project_type, annotations_filenames,
-                folder_path, annotation_classes, thread_id, chunksize,
+                folder_path, annotation_classes_dict, thread_id, chunksize,
                 num_uploaded, from_s3_bucket, actually_uploaded
             )
         )
@@ -730,8 +811,8 @@ def _upload_annotations_from_folder_to_project(
 
 def __upload_preannotations_thread(
     aws_creds, project_type, preannotations_filenames, folder_path,
-    annotation_classes, thread_id, chunksize, num_uploaded, already_uploaded,
-    from_s3_bucket
+    annotation_classes_dict, thread_id, chunksize, num_uploaded,
+    already_uploaded, from_s3_bucket
 ):
     len_preanns = len(preannotations_filenames)
     start_index = thread_id * chunksize
@@ -753,14 +834,6 @@ def __upload_preannotations_thread(
         from_session = boto3.Session()
         from_s3 = from_session.resource('s3')
 
-    annotation_classes_dict = {}
-    for annotation_class in annotation_classes:
-        if annotation_class["name"] in annotation_classes_dict:
-            logger.warning(
-                "Duplicate annotation class name %s. Only one of the annotation classes will be used. This will result in errors in annotation upload.",
-                annotation_class["name"]
-            )
-        annotation_classes_dict[annotation_class["name"]] = annotation_class
     for i in range(start_index, end_index):
         if already_uploaded[i]:
             continue
@@ -776,20 +849,7 @@ def __upload_preannotations_thread(
             file.seek(0)
             annotation_json = json.load(file)
 
-        for ann in annotation_json:
-            if (
-                "userId" in ann and "type" in ann and ann["type"] == "meta"
-            ) or "className" not in ann:
-                continue
-            annotation_class_name = ann["className"]
-            if not annotation_class_name in annotation_classes_dict:
-                logger.error(
-                    "Couldn't find annotation class %s in project's annotation classes.",
-                    annotation_class_name
-                )
-                sys.exit(1)
-            class_id = annotation_classes_dict[annotation_class_name]["id"]
-            ann["classId"] = class_id
+        fill_class_and_attribute_ids(annotation_json, annotation_classes_dict)
         bucket.put_object(
             Key=aws_creds["filePath"] + f"/{json_filename}",
             Body=json.dumps(annotation_json)
@@ -824,6 +884,24 @@ def __tqdm_thread(total_num, current_nums, finish_event):
                 break
 
 
+def __tqdm_thread_image_upload(
+    total_num, uploaded, couldnt_upload, finish_event
+):
+    with tqdm(total=total_num) as pbar:
+        while True:
+            finished = finish_event.wait(5)
+            if not finished:
+                sum_all = 0
+                for i in couldnt_upload:
+                    sum_all += len(i)
+                for i in uploaded:
+                    sum_all += len(i)
+                pbar.update(sum_all - pbar.n)
+            else:
+                pbar.update(total_num - pbar.n)
+                break
+
+
 def upload_preannotations_from_folder_to_project(
     project, folder_path, from_s3_bucket=None, recursive_subfolders=False
 ):
@@ -947,6 +1025,9 @@ def _upload_preannotations_from_folder_to_project(
     annotation_classes = search_annotation_classes(
         project, return_metadata=True
     )
+    annotation_classes_dict = get_annotation_classes_name_to_id(
+        annotation_classes
+    )
     while True:
         if sum(num_uploaded) == len_preannotations_paths:
             break
@@ -965,7 +1046,7 @@ def _upload_preannotations_from_folder_to_project(
                 target=__upload_preannotations_thread,
                 args=(
                     aws_creds, project_type, preannotations_filenames,
-                    folder_path, annotation_classes, thread_id, chunksize,
+                    folder_path, annotation_classes_dict, thread_id, chunksize,
                     num_uploaded, already_uploaded, from_s3_bucket
                 )
             )
@@ -1123,7 +1204,124 @@ def _get_upload_from_s3_bucket_to_project_status(project):
     return response.json()
 
 
-def _get_project_default_image_quality_in_editor(project):
+def get_project_workflow(project):
+    """Gets project's workflow.
+
+    Return value example: [{ "step" : <step_num>, "className" : <annotation_class>, "tool" : <tool_num>, ...},...]
+
+    :param project: project name or metadata
+    :type project: str or dict
+
+    :return: project workflow
+    :rtype: list of dicts
+    """
+    if not isinstance(project, dict):
+        project = get_project_metadata(project)
+    team_id, project_id = project["team_id"], project["id"]
+    params = {
+        "team_id": team_id,
+    }
+    response = _api.send_request(
+        req_type='GET', path=f'/project/{project_id}/workflow', params=params
+    )
+    if not response.ok:
+        raise SABaseException(
+            response.status_code,
+            "Couldn't get project workflow " + response.text
+        )
+    res = response.json()
+    annotation_classes = search_annotation_classes(
+        project, return_metadata=True
+    )
+    for r in res:
+        if "class_id" not in r:
+            continue
+        found_classid = False
+        for a_class in annotation_classes:
+            if a_class["id"] == r["class_id"]:
+                found_classid = True
+                r["className"] = a_class["name"]
+                del r["class_id"]
+                break
+        if not found_classid:
+            raise SABaseException(0, "Couldn't find class_id in workflow")
+    return res
+
+
+def set_project_workflow(project, new_workflow):
+    """Sets project's workflow.
+
+    new_workflow example: [{ "step" : <step_num>, "className" : <annotation_class>, "tool" : <tool_num>, ...},...]
+
+    :param project: project name or metadata
+    :type project: str or dict
+    :param project: new workflow list of dicts
+    :type project: list of dicts
+
+    :return: updated part of project's workflow
+    :rtype: list of dicts
+    """
+    if not isinstance(project, dict):
+        project = get_project_metadata(project)
+    if not isinstance(new_workflow, list):
+        raise SABaseException(
+            0, "Set project setting new_workflow should be a list"
+        )
+    team_id, project_id = project["team_id"], project["id"]
+
+    params = {
+        "team_id": team_id,
+    }
+    annotation_classes = search_annotation_classes(
+        project, return_metadata=True
+    )
+
+    new_list = copy.deepcopy(new_workflow)
+    for step in new_list:
+        if "id" in step:
+            del step["id"]
+        if "className" in step:
+            found = False
+            for an_class in annotation_classes:
+                if an_class["name"] == step["className"]:
+                    step["class_id"] = an_class["id"]
+                    del step["className"]
+                    found = True
+                    break
+            if not found:
+                raise SABaseException(
+                    0, "Annotation class not found in set_project_workflow."
+                )
+
+    json_req = {"steps": new_list}
+    response = _api.send_request(
+        req_type='POST',
+        path=f'/project/{project_id}/workflow',
+        params=params,
+        json_req=json_req
+    )
+    if not response.ok:
+        raise SABaseException(
+            response.status_code,
+            "Couldn't set project workflow " + response.text
+        )
+    res = response.json()
+    return res
+
+
+def get_project_settings(project):
+    """Gets project's settings.
+
+    Return value example: [{ "attribute" : "Brightness", "value" : 10, ...},...]
+
+    :param project: project name or metadata
+    :type project: str or dict
+
+    :return: project settings
+    :rtype: list of dicts
+    """
+    if not isinstance(project, dict):
+        project = get_project_metadata(project)
     team_id, project_id = project["team_id"], project["id"]
     params = {
         "team_id": team_id,
@@ -1134,54 +1332,53 @@ def _get_project_default_image_quality_in_editor(project):
     if not response.ok:
         raise SABaseException(
             response.status_code,
-            "Couldn't get project default image quality " + response.text
+            "Couldn't get project settings " + response.text
         )
-    for setting in response.json():
-        if "attribute" in setting and setting["attribute"] == "ImageQuality":
-            return setting["value"]
-    return 60
-    # raise SABaseException(
-    #     response.status_code,
-    #     "Couldn't get project default image quality " + response.text
-    # )
+    return response.json()
 
 
-def _set_project_default_image_quality_in_editor(project, quality):
+def set_project_settings(project, new_settings):
+    """Sets project's settings.
+
+    New settings format example: [{ "attribute" : "Brightness", "value" : 10, ...},...]
+
+    :param project: project name or metadata
+    :type project: str or dict
+    :param project: new settings list of dicts
+    :type project: list of dicts
+
+    :return: updated part of project's settings
+    :rtype: list of dicts
+    """
+    if not isinstance(project, dict):
+        project = get_project_metadata(project)
+    if not isinstance(new_settings, list):
+        raise SABaseException(
+            0, "Set project setting new_settings should be a list"
+        )
     team_id, project_id = project["team_id"], project["id"]
 
     params = {
         "team_id": team_id,
     }
-    response = _api.send_request(
-        req_type='GET', path=f'/project/{project_id}/settings', params=params
-    )
-    if not response.ok:
-        raise SABaseException(
-            response.status_code,
-            "Couldn't set project default image quality " + response.text
-        )
+    current_settings = get_project_settings(project)
 
-    image_quality_id = None
-    for setting in response.json():
-        if setting["attribute"] == "ImageQuality":
-            image_quality_id = setting["id"]
+    id_conv = {}
+    for setting in current_settings:
+        if "attribute" in setting:
+            id_conv[setting["attribute"]] = setting["id"]
 
-    if image_quality_id is None:
-        raise SABaseException(
-            response.status_code,
-            "Couldn't set project default image quality " + response.text
-        )
-
-    json_req = {
-        "settings":
-            [
+    new_list = []
+    for new_setting in new_settings:
+        if "attribute" in new_setting and new_setting["attribute"] in id_conv:
+            new_list.append(
                 {
-                    "id": image_quality_id,
-                    "attribute": "ImageQuality",
-                    "value": int(quality)
+                    "attribute": new_setting["attribute"],
+                    "id": id_conv[new_setting["attribute"]],
+                    "value": new_setting["value"]
                 }
-            ]
-    }
+            )
+    json_req = {"settings": new_list}
     response = _api.send_request(
         req_type='PUT',
         path=f'/project/{project_id}/settings',
@@ -1193,3 +1390,24 @@ def _set_project_default_image_quality_in_editor(project, quality):
             response.status_code,
             "Couldn't set project settings " + response.text
         )
+    return response.json()
+
+
+def _get_project_default_image_quality_in_editor(project):
+    for setting in get_project_settings(project):
+        if "attribute" in setting and setting["attribute"] == "ImageQuality":
+            return setting["value"]
+    return 60
+    # raise SABaseException(
+    #     response.status_code,
+    #     "Couldn't get project default image quality " + response.text
+    # )
+
+
+def _set_project_default_image_quality_in_editor(project, quality):
+    set_project_settings(
+        project, {
+            "attribute": "ImageQuality",
+            "value": quality
+        }
+    )
